#line 637 "report.nw"
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/times.h>
#include "heap.h"
#include "r250.h"

int main (int argc, char *argv[]) {
  int n, m, i;
  Heap *heap_ptr;
  HeapNode *newnode;
  struct tms start_tms, end_tms;
  clock_t start0, start1, end0, end1;
  double inc;

  if (3 != argc) {
    fprintf (stderr, "Usage: %s n m.\n", argv[0]);
    fprintf (stderr, "Perform hold test m times on queue of size n.\n");
    exit (1);
  }

  n = atoi (argv[1]);           /* size of queue */
  if (0 >= n) {
    fprintf (stderr, "queue size is negative!");
    exit(1);
  }
  if (1000000 <= n) {
    fprintf (stderr, "%d is too large for the queue.", n);
    exit(1);
  }

  m = atoi (argv[2]);           /* number of iterations */
  if (0 >= m) {
    fprintf (stderr, "too few heap operations");
    exit(1);
  }
  if (10000000 <= m) {
    fprintf (stderr, "too many heap operations");
    exit(1);
  }

  /* Bogus loop, for startup timings */
  newnode = (HeapNode*) malloc (sizeof (HeapNode));
  start0 = times (&start_tms);
  heap_ptr = CreateHeap (1);
  for (i = 1; i <= n; i++) {
    newnode->priority = dr250 ();
    newnode->data = i;
  }
  end0 = times (&end_tms);
  DestroyHeap (heap_ptr);

  /* Build the heap, should do some dequeues too. */
  start1 = times (&start_tms);
  heap_ptr = CreateHeap (1);
  for (i = 1; i <= n; i++) {
    newnode = (HeapNode*) malloc (sizeof(HeapNode));
    if (NULL == newnode) {
        fprintf (stderr, "no more memory for heap nodes");
        exit(1);
    }
    newnode->priority = dr250 ();
    newnode->data = i;
    Enqueue (heap_ptr, newnode);
  }
  assert (VerifyHeap (heap_ptr, 1));
  end1 = times (&end_tms);

#ifndef NDEBUG
  printf ("Start time = %ld, end time = %ld.\n", start1, end1);
  printf ("Bogus start = %ld, Bogus end = %ld.\n", start0, end0);
#endif

  printf ("Build heap elapsed time = %ld clock ticks at %d ticks/sec.\n", 
          (end1 - start1) - (end0 - start0), CLOCKS_PER_SEC);

#ifndef NDEBUG
  ShowHeap (heap_ptr);
#endif

  /* bogus timing loop */
  start0 = times (&start_tms);
  for (i = 1; i <= m; i++) {
    /* Dequeue */
    do {
      inc = dr250 ();
    } while (inc < 0.001);
    log (inc);
    /* Enqueue */
  }
  end0 = times (&end_tms);

  start1 = times (&start_tms);
  for (i = 1; i <= m; i++) {
    newnode = Dequeue (heap_ptr);
    assert (NULL != newnode);
    assert (VerifyHeap (heap_ptr, 1));
#ifndef NDEBUG
    printf ("node.priority = %f, node.data = %ld.\n", newnode->priority,
            newnode->data);
#endif
    do {
      inc = dr250 ();
    } while (inc < 0.001);
    newnode->priority -= log (inc);
    Enqueue (heap_ptr, newnode);
    assert (VerifyHeap (heap_ptr, 1));
  }
  end1 = times (&end_tms);

#ifndef NDEBUG
  ShowHeap (heap_ptr);
#endif

  printf ("HOLD elapsed time = %ld clock ticks at %d ticks/sec.\n", 
          (end1 - start1) - (end0 - start0), CLOCKS_PER_SEC);

  for (i = 1; i <= n; i++) {
    newnode = Dequeue (heap_ptr);
    free (newnode);
  }
  DestroyHeap(heap_ptr);

  return 0;
}

