#line 343 "report.nw"
/* 
#line 308 "report.nw"
Copyright (C) 1995 by Humberto Ortiz Zuazaga.
Copyright (C) 1993, 1994 by Fred Annexstein, John Franco, Humberto
Ortiz-Zuazaga, Manian Swaminathan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#line 343 "report.nw"
                        */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "heap.h"

#line 463 "report.nw"
#define Left(i) ((i) << 1)
#define Right(i) (((i) << 1) + 1)
#define Parent(i) ((i) >> 1)
#line 509 "report.nw"
void sift_down (Heap *heap, int i);


#line 410 "report.nw"
Heap *CreateHeap (size_t size) {
  Heap *heap;

  assert (0 < size);

  heap = (Heap *) malloc (sizeof (Heap));
  if (NULL != heap) {
    heap->heap_ptr = (HeapNode **) calloc (size + 1, sizeof (HeapNode *));
    if (NULL == heap->heap_ptr) {
      free (heap);
      heap = NULL;
    }
    else {
      heap->size = size;
      heap->last = 0;
    }
  }
  return heap;
}
#line 439 "report.nw"
void DestroyHeap (Heap *heap) {
  if (NULL != heap) {
    if (NULL != heap->heap_ptr) {
      while (0 < heap->last) {
        if (NULL != heap->heap_ptr[heap->last]) {
          free(heap->heap_ptr[heap->last]);
        }
        heap->last--;
      }
      free (heap->heap_ptr);
    }
    free (heap);
    heap = NULL;
  }
}
#line 475 "report.nw"
void Enqueue (Heap *heap, HeapNode *newnode) {
  int i;

  assert (NULL != heap);
  assert (NULL != newnode);

  heap->last++;
  /* Is the heap full? */
  if (heap->last > heap->size) {
    /* resize heap */
    heap->heap_ptr = realloc (heap->heap_ptr, sizeof (HeapNode *) *
                              (heap->size * 2 + 1));
    if (NULL == heap->heap_ptr) {
      fprintf (stderr, "heap: error resizing heap, cannot insert.\n");
      exit (1);
    }
    heap->size = heap->size * 2;
  }
  i = heap->last;
  while ((i > 1) && (heap->heap_ptr[Parent(i)]->priority <
                     newnode->priority)) {
    heap->heap_ptr[i] = heap->heap_ptr[Parent(i)];
    i = Parent(i);
  }
  heap->heap_ptr[i]  = newnode;
  assert (VerifyHeap(heap, 1));
}
#line 512 "report.nw"
void sift_down (Heap *heap, int i) {
  int k, j, n;

  assert (NULL != heap);
  assert (1 <= i);

  k = i;
  n = heap->last;
  do {
    j = k;
    if ((Left(j) <= n) && (heap->heap_ptr[Left(j)]->priority >
                           heap->heap_ptr[k]->priority)) {
      k = Left(j);
    }
    if ((Right(j) <= n) && (heap->heap_ptr[Right(j)]->priority >
                            heap->heap_ptr[k]->priority)) {
      k = Right(j);
    }
    heap->heap_ptr[0] = heap->heap_ptr[j];
    heap->heap_ptr[j] = heap->heap_ptr[k];
    heap->heap_ptr[k] = heap->heap_ptr[0];
  } while (j != k);
  assert (VerifyHeap (heap, i));
}
#line 546 "report.nw"
HeapNode * Dequeue (Heap *A) {
  HeapNode *max;

  assert (NULL != A);

  if (0 >= A->last) {
    fprintf (stderr, "Extracting from empty heap!\n");
    return NULL;                        /* empty heap! */
  }

  max = A->heap_ptr[1];
  A->heap_ptr[1] = A->heap_ptr[A->last];
  A->last--;
  sift_down (A, 1);             /* B+B */

  /* Check if the heap is less than a quarter full. */
  if ((A->last * 4) < A->size) {
    /* resize heap to a half */
    A->heap_ptr = realloc (A->heap_ptr, (1 + A->size / 2) *
                           sizeof(HeapNode *));
    if (A->heap_ptr == NULL) {
      fprintf (stderr, "error resizing heap.\n");
      return NULL;
    }
  }

  return max;
}
#line 584 "report.nw"
void ShowHeap (Heap *heap) {
  int i;

  assert (NULL != heap);

  puts("Heap status.");
  for  (i=1; i<= heap->last; i++)
  {
    printf ("Heap [%d] = pri %f, val = %ld\n", i,
                  heap->heap_ptr[i]->priority,
                  heap->heap_ptr[i]->data);
  }
}
#line 607 "report.nw"
int VerifyHeap (Heap *heap, int i) {
  boolean l_stat, r_stat;
  
  assert (NULL != heap);
  assert (0 < i);

  if (0 >= heap->last)
    return TRUE;                /* empty heap trivially correct */

  if (heap->last >= Left(i))
    l_stat = (heap->heap_ptr[i]->priority >= 
              heap->heap_ptr[Left(i)]->priority) &&
             VerifyHeap (heap, Left(i));
  else
    l_stat = TRUE;              /* no left children */

  if (heap->last >= Right(i))
    r_stat = (heap->heap_ptr[i]->priority >= 
              heap->heap_ptr[Right(i)]->priority) &&
             VerifyHeap (heap, Right(i));
  else
    r_stat = TRUE;              /* no right children */

  return (l_stat && r_stat);
}

